@@ -647,6 +647,16 @@ PJ_DEF(pj_status_t) pjsip_tls_transport_start2( pjsip_endpoint *endpt,
                         &listener->cert);
         if (status != PJ_SUCCESS)
             goto on_error;    
+    } else if (listener->tls_setting.cert_lookup.type !=
+                                                PJ_SSL_CERT_LOOKUP_NONE &&
+               listener->tls_setting.cert_lookup.keyword.slen)
+    {
+        status = pj_ssl_cert_load_from_store(
+                                pool,
+                                &listener->tls_setting.cert_lookup,
+                                &listener->cert);
+        if (status != PJ_SUCCESS)
+            goto on_error;
     }
 
     /* Register to transport manager */
@@ -709,7 +719,7 @@ static void lis_on_destroy(void *arg)
     }
 
     if (listener->factory.pool) {
-        PJ_LOG(4,(listener->factory.obj_name,  "SIP TLS transport destroyed"));
+        PJ_LOG(4,(listener->factory.obj_name, "SIP TLS listener destroyed"));
         pj_pool_secure_release(&listener->factory.pool);
     }
 }
@@ -756,6 +766,22 @@ PJ_DEF(pj_status_t) pjsip_tls_transport_restart(pjsip_tpfactory *factory,
     pj_status_t status = PJ_SUCCESS;
     struct tls_listener *listener = (struct tls_listener *)factory;
 
+    /* Just update the published address if currently no listener */
+    if (!listener->ssock) {
+        PJ_LOG(3,(factory->obj_name,
+                      "TLS restart requested while no listener created, "
+                      "update the published address only"));
+
+        status = update_factory_addr(listener, a_name);
+        if (status != PJ_SUCCESS)
+            return status;
+
+        /* Set transport info. */
+       update_transport_info(listener);
+
+       return PJ_SUCCESS;
+    }
+
     lis_close(listener);
 
     status = pjsip_tls_transport_lis_start(factory, local, a_name);
